<?php

/**
 *
 *   Hepsipay adına Alttantire Yazılım Çözümleri tarafından geliştirilmiştir.
 *
 * @author      Alttantire <info@alttantire.com>
 * @date        2025
 *
 */

if ( ! defined( '_PS_VERSION_' ) ) {
	exit;
}

use Alttantire\HepsipayGateway;

include_once dirname( __FILE__ ) . '/lib/HepsipayGateway.php';
require_once dirname( __FILE__ ) . '/lib/HepsipayCookieSetter.php';
require_once dirname( __FILE__ ) . '/classes/HelperModel.php';

class Hepsipay extends PaymentModule {
	protected $config_form = false;

	protected $errors = [];
	protected $confirmations = [];

	public function __construct() {
		/**
		 * Plugin config
		 */
		$this->name = 'hepsipay';
		$this->tab = 'payments_gateways';
		$this->version = '4.0.0';
		$this->author = 'Alttantire';
		$this->need_instance = 0;
		$this->bootstrap = true;
		$this->displayName = $this->l( 'Hepsipay Payment Gateway' );
		$this->description = $this->l( 'Allows you to accept payments online via any type of credit cards.' );
		$this->ps_versions_compliancy = array( 'min' => '1.6', 'max' => _PS_VERSION_ );

		/**
		 * Set cookie samesite : None
		 */
		foreach ( $_COOKIE as $k => $v ) {
			if ( $k != "PHPSESSID" ) {
				Alttantire\HepsipayCookieSetter::setcookie( $k, $v, array(
					'secure'   => true,
					'samesite' => 'None',
					'path'     => ini_get( 'session.cookie_path' ),
					'domain'   => $_SERVER['HTTP_HOST']
				) );
			}
		}

		/**
		 * Verify and Define Api Gateway
		 */
		$this->verify();

		parent::__construct();
	}

	private function verify() {

		/**
		 * Get Api Auth Settings
		 */
		$hepsipay = $this->getConfigFormValues();

		try {

			if ( ! $hepsipay["HEPSIPAY_API_URL"] ||
			     ! $hepsipay["HEPSIPAY_MERCHANT_NO"] ||
			     ! $hepsipay["HEPSIPAY_TERMINAL_NO"] ||
			     ! $hepsipay["HEPSIPAY_STORE_KEY"] ) {
				throw new \InvalidArgumentException( "" );
			}

			/**
			 * Define Api Gateway
			 */
			$this->gateway = new HepsipayGateway( $hepsipay["HEPSIPAY_API_URL"], $hepsipay["HEPSIPAY_MERCHANT_NO"],
				$hepsipay["HEPSIPAY_TERMINAL_NO"], $hepsipay["HEPSIPAY_STORE_KEY"] );

            /**
             * VerifyClient
             */
			$is_verified = Configuration::get('HEPSIPAY_VERIFIED');

			$this->errors               = array();
            $this->confirmations        = array();
            $this->verified             = $is_verified;

            if ( ! $this->verified) {
                throw new Exception("");
            } else {
                $this->confirmations[] = $this->l("Hepsipay kurulumu başarılı");
            }
		} catch ( Exception $e ) {
			$this->errors[] = "<strong>Hepsipay bilgilerinizi kontrol ediniz ve kurulumu tamamlayınız.</strong>";
		}
	}

	public function install() {
		if ( ! extension_loaded( 'curl' ) ) {
			$this->_errors[] = $this->l( 'You have to enable the cURL extension on your server to install this module' );

			return false;
		}
		Configuration::updateValue( 'HEPSIPAY_API_URL', '' );
		Configuration::updateValue( 'HEPSIPAY_MERCHANT_NO', '' );
		Configuration::updateValue( 'HEPSIPAY_TERMINAL_NO', '' );
		Configuration::updateValue( 'HEPSIPAY_STORE_KEY', '' );
		Configuration::updateValue( 'HEPSIPAY_TL_CURRENCY', '' );
		Configuration::updateValue( 'HEPSIPAY_VERIFIED', false );

		return parent::install() &&
		       $this->registerHook( 'header' ) &&
		       $this->registerHook( 'backOfficeHeader' ) &&
		       $this->registerHook( 'payment' ) &&
		       $this->registerHook( 'paymentReturn' ) &&
		       $this->registerHook( 'adminOrder' ) &&
		       $this->registerHook( 'moduleRoutes' ) &&
		       $this->registerHook( 'paymentOptions' );
	}

	public function uninstall() {
		Configuration::deleteByName( 'HEPSIPAY_API_URL' );
		Configuration::deleteByName( 'HEPSIPAY_MERCHANT_NO' );
		Configuration::deleteByName( 'HEPSIPAY_TERMINAL_NO' );
		Configuration::deleteByName( 'HEPSIPAY_STORE_KEY' );
		Configuration::deleteByName( 'HEPSIPAY_TL_CURRENCY' );
		Configuration::deleteByName( 'HEPSIPAY_VERIFIED' );

		return parent::uninstall();
	}

	/**
	 * Load the configuration form
	 */
	public function getContent() {

		/**
		 * If values have been submitted in the form, process.
		 */

		if ( ( (bool) Tools::isSubmit( 'submitHepsipayModule' ) ) ) {
			$this->postProcess();
			$this->verify();
		}

		$this->context->controller->errors = array_merge( $this->context->controller->errors, $this->errors );
		$this->context->controller->confirmations = array_merge( $this->context->controller->confirmations,
			$this->confirmations );

		/**
		 * Module
		 */

		$this->context->smarty->assign( array(
			'module_dir' => $this->_path,
		) );

		$output = $this->context->smarty->fetch( $this->local_path . 'views/templates/admin/configure.tpl' );

		return $this->renderForm() . $output;
	}

	/**
	 * Validate form fields.
	 */
	protected function validate() {

		$validate = true;

		if ( ! Tools::getValue( 'HEPSIPAY_API_URL' ) ) {
			$this->context->controller->warnings[] = $this->l( 'API URL bilgisini giriniz' );
			$validate = false;
		}
		if ( ! Tools::getValue( 'HEPSIPAY_MERCHANT_NO' ) ) {
			$this->context->controller->warnings[] = $this->l( 'Merchant No bilgisini giriniz' );
			$validate = false;
		}
		if ( ! Tools::getValue( 'HEPSIPAY_TERMINAL_NO' ) ) {
			$this->context->controller->warnings[] = $this->l( 'Terminal No bilgisini giriniz' );
			$validate = false;
		}
		if ( ! Tools::getValue( 'HEPSIPAY_STORE_KEY' ) ) {
			$this->context->controller->warnings[] = $this->l( 'Store Key bilgisini giriniz' );
			$validate = false;
		}
		if ( ! Tools::getValue( 'HEPSIPAY_TL_CURRENCY' ) ) {
			$this->context->controller->warnings[] = $this->l( 'TL para birimini seçiniz' );
			$validate = false;
		}

		if ( $validate ) {
			$posted = [
				"hepsipay_vpos_environment" => Tools::getValue( 'HEPSIPAY_API_URL' ),
				"hepsipay_vpos_merchant_no" => Tools::getValue( 'HEPSIPAY_MERCHANT_NO' ),
				"hepsipay_vpos_terminal_no" => Tools::getValue( 'HEPSIPAY_TERMINAL_NO' ),
				"hepsipay_vpos_store_key"   => Tools::getValue( 'HEPSIPAY_STORE_KEY' ),
			];

			$response = HelperModel::checkCredential( $posted, Context::getContext() );
			if ( $response === true ) {
				$validate = true;
				$this->confirmations[] = $this->l( 'Hepsipay kurulumu başarılı' );
			} else {
				$validate = false;
				$this->errors[] = "<strong>" . $response . "</strong>";
			}

		}

		return $validate;

	}

	/**
	 * Save form data.
	 */
	protected function postProcess() {
		if ( $this->validate() ) {
			$form_values = $this->getConfigFormValues();
			foreach ( array_keys( $form_values ) as $key ) {
				Configuration::updateValue( $key, Tools::getValue( $key ) );
			}

			Configuration::updateValue( 'HEPSIPAY_VERIFIED', true );

			/**
			 * VerifyClient
			 */
			$this->verify();
		}
	}

	/**
	 * Set values for the inputs.
	 */
	public function getConfigFormValues() {
		return array(
			'HEPSIPAY_API_URL'     => Configuration::get( 'HEPSIPAY_API_URL', '' ),
			'HEPSIPAY_MERCHANT_NO' => Configuration::get( 'HEPSIPAY_MERCHANT_NO', '' ),
			'HEPSIPAY_TERMINAL_NO' => Configuration::get( 'HEPSIPAY_TERMINAL_NO', '' ),
			'HEPSIPAY_STORE_KEY'   => Configuration::get( 'HEPSIPAY_STORE_KEY', '' ),
			'HEPSIPAY_TL_CURRENCY' => Configuration::get( 'HEPSIPAY_TL_CURRENCY', '' ),
//			'HEPSIPAY_VERIFIED' => Configuration::get('HEPSIPAY_VERIFIED', ''),
		);
	}

	/**
	 * Create the form that will be displayed in the configuration of your module.
	 */
	protected function renderForm() {
		$helper = new HelperForm();

		$helper->show_toolbar = true;
		$helper->table = $this->table;
		$helper->module = $this;
		$helper->default_form_language = $this->context->language->id;
		$helper->allow_employee_form_lang = Configuration::get( 'PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0 );

		$helper->identifier = $this->identifier;
		$helper->submit_action = 'submitHepsipayModule';
		$helper->currentIndex = $this->context->link->getAdminLink( 'AdminModules',
				false ) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
		$helper->token = Tools::getAdminTokenLite( 'AdminModules' );

		$helper->tpl_vars = array(
			'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
			'languages'    => $this->context->controller->getLanguages(),
			'id_language'  => $this->context->language->id,
		);

		return $helper->generateForm( array( $this->getConfigForm() ) );
	}

	/**
	 * Create the structure of your form.
	 */
	protected function getConfigForm() {
		$currencies = Currency::getCurrencies();
		$options = array();

		foreach ( $currencies as $currency ) {
			$options[] = array(
				'id_option' => $currency['id_currency'],
				'name'      => $currency['name'] . ' (' . $currency['iso_code'] . ')',
			);
		}

		return array(
			'form' => array(
				'legend' => array(
					'title' => $this->l( 'Settings' ),
					'icon'  => 'icon-cogs',
				),
				'input'  => array(

					array(
						'col'    => 3,
						'type'   => 'text',
						'prefix' => '<i class="icon icon-key"></i>',
						'name'   => 'HEPSIPAY_API_URL',
						'label'  => "API URL",
					),
					array(
						'col'    => 3,
						'type'   => 'text',
						'prefix' => '<i class="icon icon-key"></i>',
						'name'   => 'HEPSIPAY_MERCHANT_NO',
						'label'  => $this->l( 'Merchant No' ),
					),
					array(
						'col'    => 3,
						'type'   => 'text',
						'prefix' => '<i class="icon icon-key"></i>',
						'name'   => 'HEPSIPAY_TERMINAL_NO',
						'label'  => $this->l( 'Terminal No' ),
					),
					array(
						'col'    => 3,
						'type'   => 'text',
						'prefix' => '<i class="icon icon-home"></i>',
						'name'   => 'HEPSIPAY_STORE_KEY',
						'label'  => $this->l( 'Store Key' ),
					),
					array(
						'col'     => 3,
						'type'    => 'select',
						'name'    => 'HEPSIPAY_TL_CURRENCY',
						'label'   => $this->l( 'TL Para Birimi' ),
						'options' => array(
							'query' => $options,
							'id'    => 'id_option',
							'name'  => 'name',
						),
						'value'   => Configuration::get( 'HEPSIPAY_TL_CURRENCY', 1 ),
						// Varsayılan değer olarak 1. seçeneği kullan
						'desc'    => $this->l( 'Hepsipay sadece TL para biriminde çalışır.' ),
						// Açıklama
					),
					array(
						'type'  => 'html',
						'name'  => 'custom_text',
						'label' => $this->l( 'Sistem Bilgileri' ),

						'html_content' => HelperModel::renderSystemInfo(),
					),

				),

				'submit' => array(
					'title' => $this->l( 'Save' ),
				),
			),
		);
	}

	public function hookModuleRoutes() {
	}

	/**
	 * Add the CSS & JavaScript files you want to be added on the FO.
	 */
	public function hookHeader() {
		$this->context->controller->addJquery();
	}

	/**
	 * This hook is used to display the order confirmation page.
	 */
	public function hookPaymentReturn( $params ) {
		if ( ! $this->active ) {
			return null;
		}

		if ( ! version_compare( _PS_VERSION_, '1.7.0', '>=' ) ) {
			$order = $params['objOrder'];
			$this->smarty->assign( array(
				'status'    => true,
				'id_order'  => $order->id,
				'reference' => $order->reference,
				'params'    => $params,
				'total'     => Tools::displayPrice( $params['total_to_pay'], $params['currencyObj'], false ),
			) );

			return $this->display( __FILE__, 'views/templates/hook/confirmation.tpl' );

		} else {
			$order = $params['order'];
			$currency = Context::getContext()->currency;

			$this->smarty->assign( array(
				'status'    => true,
				'id_order'  => $order->id,
				'reference' => $order->reference,
				'params'    => $params,
				'total'     => Tools::displayPrice( $order->total_paid, $currency, false ),
			) );
		}
	}

	public function hookPaymentOptions( $params ) {

		if ( ! $this->active ) {
			return null;
		}
		if ( ! version_compare( _PS_VERSION_, '1.7.0', '>=' ) ) {
			return $this->hookPayment();
		}
		$payment_options = array();
		$newOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption;
		$newOption->setModuleName( $this->name )->
		setCallToActionText( $this->l( 'Pay by credit card' ) )->
		setAction( $this->context->link->getModuleLink( $this->name, 'payment', array(), true ) )->
		setLogo( Media::getMediaPath( _PS_MODULE_DIR_ . $this->name . '/views/img/logo.png' ) )->
		setAdditionalInformation( $this->l( 'Secure and Easy CC Payment for all Cards' ) );
		$payment_options[] = $newOption;

		return $payment_options;
	}

	/**
	 * This method is used to render the payment button,
	 * Take care if the button should be displayed or not.
	 */
	public function hookPayment() {

		if ( ! $this->active || ! $this->verified) {
			return null;
		}

		$this->smarty->assign( 'module_dir', $this->_path );

		return $this->display( __FILE__, 'views/templates/hook/payment.tpl' );
	}

}
